from datetime import datetime
from itertools import groupby
from typing import Optional, Tuple

from moscow_routes_parser.model import Stop, Stop_builder, Timetable_stop_time, Timetable_stop, Timetable, \
    Timetable_stop_builder, Timetable_builder


class Stop_impl(Stop):

    def get_coords(self) -> Tuple[float, float]:
        return self.coords

    def get_name(self) -> str:
        return self.name

    def __str__(self):
        return self.get_name()

    def __repr__(self):
        return self.get_name()

    def __hash__(self):
        return hash((self.get_coords(),self.name))

    def __init__(self, name: str, coords: Tuple[float, float]):
        self.coords = coords
        self.name = name

    def __eq__(self, other):
        return not (self.get_coords()[0] is None) and \
               not (self.get_coords()[1] is None) and \
               not (other.get_coords()[0] is None) and \
               not (other.get_coords()[1] is None) and            \
               self.get_name() == other.get_name() and \
               abs(self.get_coords()[0] - other.get_coords()[0]) <= 0.0000005 and \
               abs(self.get_coords()[1] - other.get_coords()[1]) <= 0.0000005

    def __ne__(self, other):
        return not (self == other)


class Stop_builder_impl(Stop_builder):

    def set_coords(self, coords: (float, float)) -> Stop_builder:
        self.coords = coords
        return self

    def __init__(self):
        self.coords = None
        self.name = None

    def set_name(self, name: str) -> Stop_builder:
        self.name = name
        return self

    def build(self) -> Stop:
        return Stop_impl(self.name, self.coords)


class Timetable_stop_time_t_mos_ru(Timetable_stop_time):

    def __init__(self, time_flight: datetime.time, color_special_flight: Optional[str]):
        self.time_flight = time_flight
        self.color_special_flight = color_special_flight

    def get_time(self) -> datetime.time:
        return self.time_flight

    def get_color_special_flight(self) -> Optional[str]:
        return self.color_special_flight

    def __eq__(self, other):
        return self.get_time() == other.get_time() and \
               self.get_color_special_flight() == other.get_color_special_flight()

    def __ne__(self, other):
        return not (self == other)

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        str_time = self.get_time().strftime("%H:%M")
        color = self.get_color_special_flight()
        if not (color is None):
            str_time += " ({})".format(color)
        return str_time


class Timetable_stop_t_mos_ru(Timetable_stop):

    def __init__(self, stop: Stop, times: list[Timetable_stop_time_t_mos_ru]):
        self.stop = stop
        self.times = times

    def get_times(self) -> iter:
        return iter(self.times)

    def get_stop(self) -> Stop:
        return self.stop

    def __eq__(self, other):
        return self.get_stop() == other.get_stop() and \
               list(sorted(self.get_times(), key=lambda x: x.get_time())) == list(
            sorted(other.get_times(), key=lambda x: x.get_time()))

    def __ne__(self, other):
        return not (self == other)

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        result = "{} ({},{}):\n".format(self.get_name(), self.get_coords()[0], self.get_coords()[1])
        time_by_hours = groupby(sorted(self.get_times(), key=lambda t: t.get_time()), key=lambda t: t.get_time().hour)
        for h in time_by_hours:
            result += str(h[0]) + ': ' + \
                      ' '.join(map(lambda t: str(t.get_time().minute) + ("" if t.get_color_special_flight() is None
                                                                         else " ({})".format(
                          t.get_color_special_flight())), h[1])) + "\n"
        return result


class Timetable_t_mos_ru(Timetable):

    def has_another_direction(self):
        return self.flag_has_another_direction

    def get_stops(self) -> list[Timetable_stop]:
        return self.stops

    def __init__(self, id_route_t_mos_ru: str, direction: int, date: datetime.date,
                 stops: list[Timetable_stop],flag_has_another_direction=False):
        self.flag_has_another_direction = flag_has_another_direction
        self.id_route_t_mos_ru = id_route_t_mos_ru
        self.direction = direction
        self.stops = stops
        self.date = date

    def get_direction(self) -> int:
        return self.direction

    def get_id_route_t_mos_ru(self) -> str:
        return self.id_route_t_mos_ru

    def get_date(self) -> datetime.date:
        return self.date

    def __iter__(self):
        return iter(self.get_stops())

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        result = "Route with id={}, direction = {}, date={}: ".format(self.get_id_route_t_mos_ru(),
                                                                      self.get_direction(), self.get_date()) + "\n"
        result += ''.join(str(self.get_stops()))
        return result


class Timetable_stop_builder_t_mos_ru(Timetable_stop_builder):

    def add_item_timetable(self, time_flight: datetime.time,
                           special_flight: Optional[str] = None) -> Timetable_stop_builder:
        self.time_flights.append(Timetable_stop_time_t_mos_ru(time_flight, special_flight))
        return self

    def set_name(self, name: str) -> Timetable_stop_builder:
        self.name = name
        return self

    def set_coords(self, coords: (float, float)) -> Timetable_stop_builder:
        self.coords = coords
        return self

    def __init__(self):
        self.coords = None
        self.name = None
        self.time_flights = []

    def build(self) -> Timetable_stop:
        stop = Stop_impl(self.name, self.coords)
        return Timetable_stop_t_mos_ru(stop, self.time_flights)


class Timetable_builder_t_mos_ru(Timetable_builder):

    def __init__(self):
        #Default True for backward compatilibity
        self.flag_has_another_direction = True
        self.date = None
        self.direction = None
        self.id_route_t_mos_ru = None
        self.stops = []

    def set_date(self, date: datetime.date) -> Timetable_builder:
        self.date = date
        return self

    def add_stop(self) -> Timetable_stop_builder:
        stop_builder = Timetable_stop_builder_t_mos_ru()
        self.stops.append(stop_builder)
        return stop_builder

    def build(self) -> Timetable:
        return Timetable_t_mos_ru(
            self.id_route_t_mos_ru,
            self.direction,
            self.date,
            list(map(lambda stop_builder: stop_builder.build(),
                     self.stops)),
            self.flag_has_another_direction
        )

    def set_id_route_t_mos_ru(self, id_route_t_mos_ru: str) -> Timetable_builder:
        self.id_route_t_mos_ru = id_route_t_mos_ru
        return self

    def set_direction(self, direction: int) -> Timetable_builder:
        self.direction = direction
        return self

    def set_has_another_direction(self,flag_has_another_direction=True):
        self.flag_has_another_direction=flag_has_another_direction





import html
import json
import logging
import re
from abc import abstractmethod
from datetime import datetime, time
from time import sleep
from typing import Optional

import requests

from moscow_routes_parser.model import Route, Timetable, Equipment, Timetable_builder
from moscow_routes_parser.model_impl import Timetable_builder_t_mos_ru


class parser_timetable:
    """"Interface for parser"""

    @abstractmethod
    def parse(self, text: str) -> Timetable_builder:
        pass


class parser_timetable_t_mos_ru(parser_timetable):
    """"Parser for timetable from t.mos.ru implementation"""

    def __init__(self, builder: Timetable_builder):
        """"Initialize parser
        :param builder: Builder for Timetable for route
        """
        self.builder = lambda: builder

    def parse(self, text: str) -> Timetable_builder:
        """Parse text from https://transport.mos.ru/ru/ajax/App/ScheduleController/getRoute (for format using
        2022-Jan-11)

        Since 12.01.2022 t.mos.ru drop data-services from results
        Since 13.03.2022 added flag_has_another_direction
        @param text: text for parse
        @return Timetable for route
        """

        result_stops = type(self.builder())()
        # stops = re.finditer(r'data-stop="([^"]*?)".*?data-services="([^"]*?)".*?d-inline.*?>(.*?)<(.*?)</li>', text,
        #                     re.M + re.S
        #                     )
        stops = re.finditer(r'data-stop="(.*?)".*?d-inline.*?>(.*?)<(.*?)</li>', text,
                            re.M + re.S
                            )
        data_coords_iter = re.finditer(r'data-coords="(.*?)"', text,
                                       re.M + re.S
                                       )
        data_coords_list = list(data_coords_iter)
        if re.search(r'ic-change-a-b', text, re.M + re.S) is None:
            result_stops.set_has_another_direction(False)
        else:
            result_stops.set_has_another_direction(True)
        # если есть расписание
        if len(data_coords_list) > 0:
            data_coords = data_coords_list[0].group(1)
            data_coords = html.unescape(data_coords)
            data_coords = json.loads(data_coords)['features']
            data_coords = iter(map(lambda feature: feature['geometry']['coordinates'], data_coords))
        else:
            data_coords = []

        for stop in stops:
            name_stop = stop.group(2)
            coords_stop = next(data_coords)
            description = stop.group(3)
            logger = logging.getLogger(__name__)
            logger.info(name_stop)
            hours = re.finditer(r'dt1.*?(\d\d):(.*?)</div>\s*</div>\s*</div>', description, re.M + re.S)
            timetable_stop = result_stops.add_stop()
            timetable_stop.set_name(name_stop)
            timetable_stop.set_coords(coords_stop)
            log_timetable = ""
            for hour in hours:
                num_hour = int(hour.group(1))
                minutes_text = hour.group(2)
                log_timetable += str(num_hour) + ": "
                minutes = re.finditer(r'div10([^>]*)>\s*(\d\d)', minutes_text, re.M + re.S)
                for minute in minutes:
                    num_minute = int(minute.group(2))
                    color_start = minute.group(1).find('color: ')
                    if color_start >= 0:
                        quote = minute.group(1).find('"', color_start)
                        min_color = minute.group(1)[color_start + 7:quote]
                    else:
                        min_color = None
                    if not (min_color is None):
                        log_timetable += "{}{}".format(num_minute, min_color) + " "
                        pass
                    else:
                        log_timetable += str(num_minute) + " "
                        pass
                    time_flight = time(num_hour, num_minute)
                    timetable_stop.add_item_timetable(time_flight, min_color)
                logger.info(log_timetable)
        return result_stops


class Parser_routes:

    @abstractmethod
    def parse(self, text: str) -> [Route]:
        pass


class Parser_routes_t_mos_ru(Parser_routes):

    def __init__(self):
        self.count = None

    def parse(self, text: str) -> [Route]:
        """"Parses route info from transport.mos.ru (name, id, type)
        :param text: text for parsing from t.mos.ru
        :return list of Route
        """
        count_result = re.finditer(r'data-count-pages="(\d+)"', text, re.M + re.S)
        self.count = int(list(count_result)[0].group(1))

        result = re.finditer(r'<a.*?href=.*?route/(.+?)".*?<div.*?ic[ ]([a-z-]+).*?</i>\s*(\S+?)\s*</div>', text,
                             re.M + re.S)
        list_routes = []

        for route in result:
            num = route.group(1)
            type_route = route.group(2)
            if type_route.find('-bus') >= 0:
                type_route = Equipment.bus()
            elif type_route.find('tramway') >= 0:
                type_route = Equipment.tramway()
            elif type_route.find('trolleybus') >= 0:
                type_route = Equipment.trolleybus()
            else:
                logging.getLogger(__name__).error("Unknown type route: {}".format(type_route))
                type_route = None
            name = route.group(3)
            list_routes.append(Route(num, type_route, name))
        return list_routes


def get_route(date: datetime.date, id_route_t_mos_ru: str, direction: int,
              get_route_url: str = 'https://transport.mos.ru/transport/schedule/route/',
              parser: parser_timetable = parser_timetable_t_mos_ru(builder=Timetable_builder_t_mos_ru())
              ) -> Timetable:
    """Get timetable for route by date and direction
        :param date: date of timetable for route
        :param id_route_t_mos_ru: id of route from t.mos.ru
        :param direction: direction for route (0 or 1)
        :param get_route_url URL for requesting timetable
        :param parser for timetable
        :return timetable for route by date and direction
    """
    logger = logging.getLogger(__name__)
    try:
        # strange problem with SSL Cert in package
        response = requests.get(get_route_url,
                                params={
                                    'mgt_schedule[isNight]': '',
                                    'mgt_schedule[date]': date.strftime("%d.%m.%Y"),
                                    'mgt_schedule[route]': id_route_t_mos_ru,
                                    'mgt_schedule[direction]': direction,
                                },
                                headers={'X-Requested-With': 'XMLHttpRequest',
                                         'User-Agent': 'Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Mobile Safari/537.36'
                                         }
                                )
        if response.status_code == 200:
            logger.info("Get route #{}".format(id_route_t_mos_ru))
            route_info = parser.parse(response.text)
        else:
            logger.error("Error status: {}".format(response.status_code))
            route_info = None
    except requests.exceptions.RequestException as e:
        logger.error("Error " + str(e))
        route_info = None

    if not (route_info is None):
        result = route_info.set_id_route_t_mos_ru(id_route_t_mos_ru).set_direction(direction).set_date(date).build()
        if len(result.get_stops()) == 0:  # Error of loading timetable without exceptions
            result = None
    else:
        result = None
    return result


def get_list_routes(work_time: int, direction: int,
                    parser: Parser_routes = None,
                    get_routes_url: str = 'https://transport.mos.ru/ru/ajax/App/ScheduleController/getRoutesList'
                    ) -> Optional[list[Route]]:
    """get list routes by work_time and direction from transport.mos.ru
        :param parser: function to parse got string
        :param get_routes_url: url for requesting routes
        :param work_time: work day or not (1 or 0)
        :param direction: 0
        :return list of Route
    """
    if parser is None:
        parser = Parser_routes_t_mos_ru()
    page = 1
    result_routes = []
    finish = False
    count = None
    logger = logging.getLogger(__name__)
    while not finish:
        finish = False
        repeat = True
        while repeat:
            repeat = False
            try:
                # strange problem with SSL Cert in package
                response = requests.get(get_routes_url,
                                        params={
                                            'mgt_schedule[search]': '',
                                            'mgt_schedule[isNight]': '',
                                            #  'mgt_schedule[filters]': '',
                                            'mgt_schedule[work_time]': work_time,
                                            'page': page,
                                            'mgt_schedule[direction]': direction,
                                        }
                                        , headers={'X-Requested-With': 'XMLHttpRequest',
                                                   'User-Agent': 'Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Mobile Safari/537.36'
                                                   }
                                        # , headers={'Cookie': "_ym_d=1637468102; _ym_uid=1637468102592825648; mos_id=rBEAAmGaFNawBwAOHRgWAgA=; _ga=GA1.2.1733238845.1637487830; uxs_uid=147e2110-500d-11ec-a7cb-8bb8b12c3186; KFP_DID=ee285837-cd1f-0a9b-c8a2-9cef6a4ee333; _ym_isad=2; _ym_visorc=w"}
                                        )
                if response.status_code == 200:
                    logger.info("Get page #{}".format(page))
                    routes = parser.parse(response.text)
                    result_routes += routes
                    if count is None:
                        count = parser.count
                    if not routes:
                        finish = True
                else:
                    logger.error("Error status: {}".format(response.status_code))
                    finish = True
                page = page + 1
                if page > count:
                    finish = True
                if not finish:
                    sleep(5)
            except requests.exceptions.RequestException as e:
                logger.error("Error " + str(e))
                repeat = True

    return result_routes



from unittest import TestCase

from moscow_routes_parser.model import Equipment, Route


class TestEquipment(TestCase):
    def test_equipment_tramway(self):
        equipment = Equipment.tramway()
        self.assertEqual(equipment.to_number(), Equipment.tramway().to_number())
        self.assertEqual(Equipment.by_number(equipment.to_number()), equipment)
        self.assertEqual(equipment, Equipment.tramway())

    def test_equipment_bus(self):
        equipment = Equipment.bus()
        self.assertEqual(equipment.to_number(), Equipment.bus().to_number())
        self.assertEqual(Equipment.by_number(equipment.to_number()), equipment)
        self.assertEqual(equipment, Equipment.bus())

    def test_equipment_trolleybus(self):
        equipment = Equipment.trolleybus()
        self.assertEqual(equipment.to_number(), Equipment.trolleybus().to_number())
        self.assertEqual(Equipment.by_number(equipment.to_number()), equipment)
        self.assertEqual(equipment, Equipment.trolleybus())


    def test_str(self):
        self.assertEqual(Equipment.tramway().to_str(), "tramway")
        self.assertEqual(Equipment.trolleybus().to_str(), "trolleybus")
        self.assertEqual(Equipment.bus().to_str(), "bus")

    def test_ne(self):
        self.assertTrue(Equipment.tramway() != Equipment.bus())
        self.assertTrue(Equipment.tramway() != Equipment.trolleybus())
        self.assertTrue(Equipment.bus() != Equipment.trolleybus())

    def test_hash(self):
        self.assertTrue(hash(Equipment.tramway()) != hash(Equipment.bus()))
        self.assertTrue(hash(Equipment.tramway()) != hash(Equipment.trolleybus()))
        self.assertTrue(hash(Equipment.bus()) != hash(Equipment.trolleybus()))
        self.assertTrue(hash(Equipment.tramway()) == hash(Equipment.tramway()))
        self.assertTrue(hash(Equipment.trolleybus()) == hash(Equipment.trolleybus()))
        self.assertTrue(hash(Equipment.bus()) == hash(Equipment.bus()))



class TestRoute(TestCase):
    def test_route(self):
        route = Route('937', Equipment.tramway(), 'A')
        self.assertEqual(route.get_equipment(), Equipment.tramway())
        self.assertEqual(route.get_name(), "A")
        self.assertEqual(route.get_id_mgt(), "937")

    def test_eq(self):
        route = Route('937', Equipment.tramway(), 'A')
        route2 = Route('937', Equipment.tramway(), 'A')
        self.assertTrue(route == route2)
        route3 = Route('936', Equipment.tramway(), 'A')
        route4 = Route('937', Equipment.bus(), 'A')
        route5 = Route('937', Equipment.tramway(), 'A2')
        self.assertFalse(route == route3)
        self.assertFalse(route == route4)
        self.assertFalse(route == route5)

    def test_ne(self):
        route = Route('937', Equipment.tramway(), 'A')
        route2 = Route('937', Equipment.tramway(), 'A')
        self.assertFalse(route != route2)
        route3 = Route('936', Equipment.tramway(), 'A')
        route4 = Route('937', Equipment.bus(), 'A')
        route5 = Route('937', Equipment.tramway(), 'A2')
        self.assertTrue(route != route3)
        self.assertTrue(route != route4)
        self.assertTrue(route != route5)

    def test_list(self):
        route = Route('937', Equipment.tramway(), 'A')
        route2 = Route('937', Equipment.tramway(), 'A')
        self.assertTrue(route in [route2])
        route3 = Route('936', Equipment.tramway(), 'A')
        self.assertFalse(route in [route3])



from datetime import time, date
from unittest import TestCase

from moscow_routes_parser.model_impl import Stop_impl, Stop_builder_impl, Timetable_stop_time_t_mos_ru, \
    Timetable_t_mos_ru, Timetable_stop_builder_t_mos_ru, Timetable_builder_t_mos_ru


class TestStop_impl(TestCase):
    def test_stop(self):
        name = "Stop"
        coords = (10.2, 13.3)
        stop = Stop_impl(name, coords)

        self.assertEqual(stop.get_name(), name)
        self.assertEqual(stop.get_coords(), coords)

    def test_eq(self):
        name = "Stop"
        coords = (37.574604,55.499970)
        name2 = "Stop"
        coords2 = (37.5746041,55.4999701)
        name3 = "Stop"
        coords3 = (37.574605,55.499971)
        stop = Stop_impl(name,coords)
        stop2 = Stop_impl(name2,coords2)
        stop3 = Stop_impl(name3,coords3)
        self.assertTrue(stop==stop2)
        self.assertFalse(stop==stop3)

class TestStop_builder_impl(TestCase):
    def test_builder(self):
        name = "Stop"
        stop = Stop_builder_impl().set_name(name).set_coords((10, 20)).build()

        self.assertEqual(stop.get_name(), name)
        self.assertEqual(stop.get_coords(), (10, 20))

    def test_builder_reverse(self):
        name = "StopStop"
        stop = Stop_builder_impl().set_coords((10, 20)).set_name(name).build()

        self.assertEqual(stop.get_name(), name)
        self.assertEqual(stop.get_coords(), (10, 20))


class TestTimetable_stop_time_t_mos_ru(TestCase):
    def test_timetable_stop_time(self):
        t = time(10, 15)
        color = "Red"
        timetable_stop_time = Timetable_stop_time_t_mos_ru(t, color)

        self.assertEqual(timetable_stop_time.get_time(), t)
        self.assertEqual(timetable_stop_time.get_color_special_flight(), color)

    def test_timetable_stop_time_none_color(self):
        t = time(10, 15)
        color = None
        timetable_stop_time = Timetable_stop_time_t_mos_ru(t, color)

        self.assertEqual(timetable_stop_time.get_time(), t)
        self.assertEqual(timetable_stop_time.get_color_special_flight(), color)

    def test_timetable_stop_time_equal_true(self):
        t = time(10, 15)
        color = "Red"
        timetable_stop_time = Timetable_stop_time_t_mos_ru(t, color)
        t2 = time(10, 15)
        color2 = "Red"
        timetable_stop_time2 = Timetable_stop_time_t_mos_ru(t2, color2)
        self.assertTrue(timetable_stop_time == timetable_stop_time2)
        self.assertFalse(timetable_stop_time != timetable_stop_time2)

    def test_timetable_stop_time_equal_false_color(self):
        t = time(10, 15)
        color = "Blue"
        timetable_stop_time = Timetable_stop_time_t_mos_ru(t, color)
        t2 = time(10, 15)
        color2 = "Red"
        timetable_stop_time2 = Timetable_stop_time_t_mos_ru(t2, color2)
        self.assertFalse(timetable_stop_time == timetable_stop_time2)
        self.assertTrue(timetable_stop_time != timetable_stop_time2)

    def test_timetable_stop_time_equal_false_time(self):
        t = time(10, 16)
        color = "Red"
        timetable_stop_time = Timetable_stop_time_t_mos_ru(t, color)
        t2 = time(10, 15)
        color2 = "Red"
        timetable_stop_time2 = Timetable_stop_time_t_mos_ru(t2, color2)
        self.assertFalse(timetable_stop_time == timetable_stop_time2)
        self.assertTrue(timetable_stop_time != timetable_stop_time2)

    def test_timetable_stop_time_equal_false_time_color(self):
        t = time(10, 16)
        color = "Blue"
        timetable_stop_time = Timetable_stop_time_t_mos_ru(t, color)
        t2 = time(10, 15)
        color2 = "Red"
        timetable_stop_time2 = Timetable_stop_time_t_mos_ru(t2, color2)
        self.assertFalse(timetable_stop_time == timetable_stop_time2)
        self.assertTrue(timetable_stop_time != timetable_stop_time2)

    def test_timetable_stop_time_str_without_color(self):
        t = time(23, 59)
        color = None
        timetable_stop_time = Timetable_stop_time_t_mos_ru(t, color)
        self.assertEqual(str(timetable_stop_time), "23:59")
        self.assertEqual(repr(timetable_stop_time), "23:59")

    def test_timetable_stop_time_str_with_color(self):
        t = time(1, 1)
        color = "Red"
        timetable_stop_time = Timetable_stop_time_t_mos_ru(t, color)
        self.assertEqual(str(timetable_stop_time), "01:01 (Red)")
        self.assertEqual(repr(timetable_stop_time), "01:01 (Red)")


# Timetable_stop don't use without builder

# class TestTimetable_stop_t_mos_ru(TestCase):
#     def test_timetable_stop(self):
#         name = "Улица Иванова"
#         times = [
#             Timetable_stop_time_t_mos_ru(time(10, 12), "Red"),
#             Timetable_stop_time_t_mos_ru(time(20, 30), None),
#         ]
#         stop = Timetable_stop_t_mos_ru(name, [10, 20], times)
#         self.assertEqual(stop.get_name(), name)
#         self.assertEqual(list(stop.get_times()), times)
#         self.assertEqual(stop.get_coords(), [10, 20])
#
#     def test_eq(self):
#         name = "Улица Иванова"
#         times = [
#             Timetable_stop_time_t_mos_ru(time(10, 12), "Red"),
#             Timetable_stop_time_t_mos_ru(time(20, 30), None),
#         ]
#         stop = Timetable_stop_t_mos_ru(name, [10, 20], times)
#         name2 = "Улица Иванова"
#         times2 = [
#             Timetable_stop_time_t_mos_ru(time(10, 12), "Red"),
#             Timetable_stop_time_t_mos_ru(time(20, 30), None),
#         ]
#         stop2 = Timetable_stop_t_mos_ru(name2, [10, 20], times2)
#         self.assertTrue(stop == stop2)
#         self.assertFalse(stop != stop2)
#
#     def test_eq_false_name(self):
#         name = "Улица Иванова"
#         times = [
#             Timetable_stop_time_t_mos_ru(time(10, 12), "Red"),
#             Timetable_stop_time_t_mos_ru(time(20, 30), None),
#         ]
#         stop = Timetable_stop_t_mos_ru(name, [10, 20], times)
#         name2 = "Улица Ивановаа"
#         times2 = [
#             Timetable_stop_time_t_mos_ru(time(10, 12), "Red"),
#             Timetable_stop_time_t_mos_ru(time(20, 30), None),
#         ]
#         stop2 = Timetable_stop_t_mos_ru(name2, [10, 20], times2)
#         self.assertFalse(stop == stop2)
#         self.assertTrue(stop != stop2)
#
#     def test_eq_false_time(self):
#         name = "Улица Иванова"
#         times = [
#             Timetable_stop_time_t_mos_ru(time(10, 12), "Red"),
#             Timetable_stop_time_t_mos_ru(time(20, 30), None),
#         ]
#         stop = Timetable_stop_t_mos_ru(name, [10, 20], times)
#         name2 = "Улица Иванова"
#         times2 = [
#             Timetable_stop_time_t_mos_ru(time(10, 13), "Red"),
#             Timetable_stop_time_t_mos_ru(time(20, 30), None),
#         ]
#         stop2 = Timetable_stop_t_mos_ru(name2, [10, 20], times2)
#         self.assertFalse(stop == stop2)
#         self.assertTrue(stop != stop2)
#
#     def test_eq_false_coords(self):
#         name = "Улица Иванова"
#         times = [
#             Timetable_stop_time_t_mos_ru(time(10, 12), "Red"),
#             Timetable_stop_time_t_mos_ru(time(20, 30), None),
#         ]
#         stop = Timetable_stop_t_mos_ru(name, [10, 20], times)
#         name2 = "Улица Иванова"
#         times2 = [
#             Timetable_stop_time_t_mos_ru(time(10, 12), "Red"),
#             Timetable_stop_time_t_mos_ru(time(20, 30), None),
#         ]
#         stop2 = Timetable_stop_t_mos_ru(name2, [10, 21], times2)
#         self.assertFalse(stop == stop2)
#         self.assertTrue(stop != stop2)
#
#     def test_eq_shake(self):
#         name = "Улица Иванова"
#         times = [
#             Timetable_stop_time_t_mos_ru(time(10, 12), "Red"),
#             Timetable_stop_time_t_mos_ru(time(20, 30), None),
#         ]
#         stop = Timetable_stop_t_mos_ru(name, [10, 20], times)
#         name2 = "Улица Иванова"
#         times2 = [
#             Timetable_stop_time_t_mos_ru(time(20, 30), None),
#             Timetable_stop_time_t_mos_ru(time(10, 12), "Red"),
#         ]
#         stop2 = Timetable_stop_t_mos_ru(name2, [10, 20], times2)
#         self.assertTrue(stop == stop2)
#         self.assertFalse(stop != stop2)


class TestTimetable_t_mos_ru(TestCase):

    def test_timetable_t_mos_ru(self):
        id_route_t_mos_ru = '393'
        direction = 1
        current_date = date(2022, 2, 20)
        times = [
            Timetable_stop_time_t_mos_ru(time(10, 12), "Red"),
            Timetable_stop_time_t_mos_ru(time(20, 30), None),
        ]
        stop = Timetable_stop_builder_t_mos_ru().set_name('ул. Иванова').set_coords([20, 30]). \
            add_item_timetable(times[0].get_time(), times[0].get_color_special_flight()). \
            add_item_timetable(times[1].get_time()).build()

        stops = [
            stop
        ]

        timetable = Timetable_t_mos_ru(id_route_t_mos_ru, direction, current_date, stops)
        self.assertEqual(timetable.get_id_route_t_mos_ru(), id_route_t_mos_ru)
        self.assertEqual(timetable.get_date(), current_date)
        self.assertEqual(timetable.get_stops(), stops)
        self.assertEqual(timetable.get_direction(), direction)
        for stop_current in timetable:
            self.assertEqual(stop_current, stop)


class TestTimetable_stop_builder_t_mos_ru(TestCase):
    def test_timetable_stop(self):
        name = "Улица Иванова"
        times = [
            Timetable_stop_time_t_mos_ru(time(10, 12), "Red"),
            Timetable_stop_time_t_mos_ru(time(20, 30), None),
        ]
        stop = Timetable_stop_builder_t_mos_ru().set_name(name). \
            add_item_timetable(times[0].get_time(), times[0].get_color_special_flight()). \
            add_item_timetable(times[1].get_time()). \
            set_coords([10, 20]).build()
        self.assertEqual(stop.get_name(), name)
        self.assertEqual(list(stop.get_times()), times)
        self.assertEqual(stop.get_coords(), [10, 20])

    pass


class TestTimetable_builder_t_mos_ru(TestCase):
    def test_timetable_builder_t_mos_ru(self):
        id_route_t_mos_ru = '393'
        direction = 1
        current_date = date(2022, 2, 20)
        times = [
            Timetable_stop_time_t_mos_ru(time(10, 12), "Red"),
            Timetable_stop_time_t_mos_ru(time(20, 30), None),
        ]
        stop = Timetable_stop_builder_t_mos_ru().set_name('ул. Иванова').set_coords([20, 30]). \
            add_item_timetable(times[0].get_time(), times[0].get_color_special_flight()). \
            add_item_timetable(times[1].get_time()).build()
        stops = [
            stop
        ]

        builder = Timetable_builder_t_mos_ru(). \
            set_date(current_date).set_direction(direction). \
            set_id_route_t_mos_ru(id_route_t_mos_ru)
        builder.add_stop().set_name('ул. Иванова').set_coords([20, 30]). \
            add_item_timetable(times[0].get_time(), times[0].get_color_special_flight()). \
            add_item_timetable(times[1].get_time())
        timetable = builder.build()
        self.assertEqual(timetable.get_id_route_t_mos_ru(), id_route_t_mos_ru)
        self.assertEqual(timetable.get_date(), current_date)
        self.assertEqual(timetable.get_stops(), stops)
        self.assertEqual(timetable.get_direction(), direction)
        for stop_current in timetable:
            self.assertEqual(stop_current, stop)

    pass



import setuptools
from setuptools import setup

setup(
    name='moscow_routes_parser',
    version='0.0.18',
    url='https://github.com/rscprof/moscow_routes_parser',
    license='MIT',
    author='rscprof',
    author_email='rscprof@gmail.com',
    description='Parser for t.mos.ru to get routes and timetables of buses, tramways and trolleybus',
    packages=["moscow_routes_parser"],
    requires=['requests']
)



""""Abstract classes for model of timetable and concrete classes for route and equipment"""
from abc import abstractmethod
from datetime import datetime
from typing import Optional


class Equipment:
    """"encapsulates equipment for routes (bus,tramway or trolleybus)"""

    def __init__(self, type_equipment: int):
        """"Constructor of equipment by int (0 - bus, 1 - tramway, 2 - trolleybus) """
        self.number = type_equipment

    def to_number(self) -> int:
        """Return number by type: 0 - bus, 1 - tramway, 2 - trolleybus"""
        return self.number

    @staticmethod
    def bus() -> 'Equipment':
        """"return Equipment object for bus"""
        return Equipment(0)

    @staticmethod
    def tramway() -> 'Equipment':
        """"return Equipment object for tramway"""
        return Equipment(1)

    @staticmethod
    def trolleybus() -> 'Equipment':
        """"return Equipment object for trolleybus"""
        return Equipment(2)

    def to_str(self) -> str:
        """"return english word for type of equipment"""
        if self.to_number() == 0:
            return "bus"
        if self.to_number() == 1:
            return "tramway"
        if self.to_number() == 2:
            return "trolleybus"

    @classmethod
    def by_number(cls, num: int) -> 'Equipment':
        """"Return Equipment object by int (0 - bus, 1 - tramway, 2 - trolleybus)
        """
        if num == 0:
            return cls.bus()
        if num == 1:
            return cls.tramway()
        if num == 2:
            return cls.trolleybus()

    def __eq__(self, other):
        """"Equipment is equal if it has same type (bus, tramway, trolleybus)"""
        return self.to_number() == (other.to_number())

    def __ne__(self, other):
        """"Equipment is not equal if it hasn't same type (bus, tramway, trolleybus)"""
        return not (self == other)

    def __hash__(self):
        """"Equipment hash is calculating by int for equipment (0 - bus, 1 - tramway, 2 - trolleybus)"""
        return hash(self.to_number())


class Route:
    """"encapsulates routes"""

    def __init__(self, id_mgt: str, equipment: Equipment, name: str):
        """"Construct Route object by
        :param id_mgt id route on t.mos.ru
        :param equipment type of route (Equipment.bus(), Equipment.tramway() or Equipment.trolleybus())
        :param name name of route
        """
        self.name = name
        self.equipment = equipment
        self.id_mgt = id_mgt

    def get_id_mgt(self) -> str:
        """"return id on t.mos.ru"""
        return self.id_mgt

    def get_equipment(self) -> Equipment:
        """"return type of Equipment"""
        return self.equipment

    def get_name(self) -> str:
        """"return name of route"""
        return self.name

    def __eq__(self, other):
        """"Routes is equal if they have same id, type of equipment and name"""
        return str(self.get_id_mgt()) == str(other.get_id_mgt()) and \
               self.get_equipment() == other.get_equipment() and \
               self.get_name() == other.get_name()
        # str() for old int type of field and changing format on t.mos.ru

    def __ne__(self, other):
        """"Routes is not equal if they haven't same id, type of equipment and name"""
        return not (self == other)

    def __hash__(self):
        """"Hash generated by id on t.mos.ru, type of equipment and name of route"""
        return hash((self.get_id_mgt(), self.get_equipment(), self.get_name()))

    def __str__(self):
        """"String representation"""
        return "{} ({}, id={})".format(self.get_name(), self.get_equipment().to_str(), self.get_id_mgt())

    def __repr__(self):
        """"String representation"""
        return str(self)


class Stop:
    """"Stop of transport (abstract)"""

    @abstractmethod
    def get_name(self) -> str:
        """"return name of stop"""
        pass

    @abstractmethod
    def get_coords(self) -> (float, float):
        """"return GPS coordinates of stop"""
        pass


class Stop_builder:
    """"Builder for stop"""

    @abstractmethod
    def set_name(self, name: str) -> 'Stop_builder':
        """Set name of stop
        :param name of stop
        """
        pass

    @abstractmethod
    def set_coords(self, coords: (float,float)) -> 'Stop_builder':
        """"Set GPS-coordinate of stop
        :param coords GPS-coordinate
        """
        pass

    @abstractmethod
    def build(self) -> Stop:
        """Build of Stop"""
        pass


class Timetable_stop:
    """"Timetable for stop for route"""

    def get_name(self) -> str:
        """"Return name of route """
        return self.get_stop().get_name()

    def get_coords(self) -> (float, float):
        """"Return coords of stop"""
        return self.get_stop().get_coords()

    @abstractmethod
    def get_times(self) -> iter:
        """"Return iterator for times of equipment being on stop"""
        pass

    @abstractmethod
    def get_stop(self) -> Stop:
        """"Return stop"""
        pass


class Timetable:
    """"Timetable of route"""

    def has_another_direction(self):
        """"Return true if the route has another direction
        By default true for backward compatible
        """
        return True

    @abstractmethod
    def __iter__(self):
        """Iterator for stops (with timetables on stop)"""
        pass

    @abstractmethod
    def get_direction(self) -> int:
        """"Return direction (0 - direct, 1 - opposite)"""
        pass

    @abstractmethod
    def get_id_route_t_mos_ru(self) -> str:
        """"Return id of route on t.mos.ru"""
        pass

    @abstractmethod
    def get_date(self) -> datetime.date:
        """"Return date of timetable"""
        pass

    @abstractmethod
    def get_stops(self) -> list[Timetable_stop]:
        """"Return timetables of stops"""
        pass


class Timetable_stop_time:
    """"Builder for timetable for stop on route"""
    @abstractmethod
    def get_time(self) -> datetime.time:
        """"Return time of arriving of equipment"""
        pass

    @abstractmethod
    def get_color_special_flight(self) -> Optional[str]:
        """"Return color of special race"""
        pass


class Timetable_stop_builder:
    """"Builder for timetable for stop"""

    @abstractmethod
    def add_item_timetable(self, time_flight: datetime.time,
                           special_flight: Optional[str] = None) -> 'Timetable_stop_builder':
        """"Adding time and color for special race
        :param time_flight time of arriving of equipment
        :param special_flight optional color of special races
        """
        pass

    @abstractmethod
    def build(self) -> Timetable_stop:
        """"Build of timetable for stop"""
        pass

    @abstractmethod
    def set_name(self, name: str) -> 'Timetable_stop_builder':
        """"Set name of stop
        :param name of stop
        """
        pass

    @abstractmethod
    def set_coords(self, coords: (float, float)) -> 'Timetable_stop_builder':
        """"Set GPS coordinates for stop"""
        pass


class Timetable_builder:
    """"Builder for timetable"""
    @abstractmethod
    def add_stop(self) -> Timetable_stop_builder:
        """"Add stop to timetable, return stop builder"""
        pass

    @abstractmethod
    def build(self) -> Timetable:
        """"Build timetable"""
        pass


    def set_has_another_direction(self,flag_has_another_direction=True):
        """"Set if the route has another direction or not
            By default nothing for backward compatibility
        """
        pass

    @abstractmethod
    def set_id_route_t_mos_ru(self, id_route_t_mos_ru: str) -> 'Timetable_builder':
        """"Set id from t.mos.ru for route
        :param id_route_t_mos_ru id from t.mos.ru
        """
        pass

    @abstractmethod
    def set_direction(self, direction: int) -> 'Timetable_builder':
        """"Set direction
        :param direction 0 - direct, 1 - opposite
        """
        pass

    @abstractmethod
    def set_date(self, date: datetime.date) -> 'Timetable_builder':
        """"Set date for timetable
        :param date for timetable
        """
        pass
